import random, string, time
from importlib import import_module


def generate_random_string(length=22):
    population = string.ascii_letters + string.digits
    choices = random.choices(population, k=length)
    random_string = ''.join(choices)
    return random_string


def serialize_user(user_data):
    username = user_data.get('username')
    if username:
        username_part = f'@{username}'
    else:
        username_part = 'no username'

    user_str = f'{user_data["first_name"]} {user_data["last_name"]} ({username_part})'
    return user_str


def message_contain(msg, words):
    msg_words = msg.lower().split()
    for msg_word in msg_words:
        for word in words:
            if msg_word.startswith(word):
                return True


def message_starts(msg, words):
    msg = msg.lower()
    for word in words:
        if msg.startswith(word):
            return True


def frange(start, stop=None, step=None):
    while stop > start:
        yield start
        start += step


def get_current_unixtime():
    return int(time.time())


def import_string(dotted_path):
    module_path, class_name = dotted_path.rsplit('.', 1)
    module = import_module(module_path)
    return getattr(module, class_name)



import pymongo

import settings
from meetg.utils import serialize_user
from meetg.loging import get_logger


logger = get_logger()


class MongoStorage(object):
    """Wrapper for MongoDB collection methods"""

    def __init__(self, db_name, collection_name):
        self.client = pymongo.MongoClient()
        self.db_name = db_name
        self.collection_name = collection_name
        self.db = getattr(self.client, db_name)
        self.collection = getattr(self.db, collection_name)

    def create(self, entry):
        return self.collection.insert_one(entry)

    def update(self, pattern, update):
        return self.collection.update_many(pattern, update)

    def update_one(self, pattern, update):
        return self.collection.update_one(pattern, update)

    def count(self, pattern=None):
        return self.collection.count(pattern)

    def find(self, pattern=None):
        return self.collection.find(pattern)

    def find_one(self, pattern=None):
        return self.collection.find_one(pattern)

    def delete(self, pattern):
        return self.collection.delete_many(pattern)

    def delete_one(self, pattern):
        return self.collection.delete_one(pattern)

    def drop(self):
        return self.db.drop_collection(self.collection_name)


class Database:
    """High-level methods to read/write data"""

    _user_fields = (
        'username', 'first_name', 'last_name', 'is_bot', 'language_code', 'phone_number', 'lat',
        'lon', 'map_link', 'notification_msg_id',
    )

    def __init__(self):
        db_name = settings.mongo_db_name
        user_collection_name = settings.mongo_user_collection_name
        self._user_st = MongoStorage(db_name, user_collection_name)

    def drop_all(self):
        self._user_st.drop()

    def create_user(
            self, chat_id, username=None, first_name=None, last_name=None, is_bot=None,
            language_code=None, phone_number=None, lat=None, lon=None, map_link=None,
            notification_msg_id=None,
        ):
        user_data = {
            'chat_id': chat_id, 'username': username, 'first_name': first_name,
            'last_name': last_name, 'is_bot': is_bot, 'language_code': language_code,
            'phone_number': phone_number, 'lat': lat, 'lon': lon, 'map_link': map_link,
            'notification_msg_id': notification_msg_id,
        }
        self._user_st.create(user_data)
        logger.info('User %s added to DB', chat_id)
        logger.debug('id %s is user %s', chat_id, serialize_user(user_data))
        return user_data

    def create_user_from_tg_user_obj(self, tg_user):
        user_data = {
            'chat_id': tg_user.id,
            'username': tg_user.username,
            'first_name': tg_user.first_name,
            'last_name': tg_user.last_name,
            'is_bot': tg_user.is_bot,
            'language_code': tg_user.language_code,
        }
        user = self.create_user(**user_data)
        return user

    def update_user(self, chat_id, **kwargs):
        # check kwargs are valid user fields
        new_data = {kwa: kwargs[kwa] for kwa in kwargs if kwa in self._user_fields}

        if new_data.get('lat') and new_data.get('lon'):
            new_data['map_link'] = (
                f"https://maps.google.com/maps?q={new_data['lat']},{new_data['lon']}"
            )

        result = self._user_st.update_one({'chat_id': chat_id}, {'$set': new_data})
        user = self.get_user(chat_id)
        logger.info('User %s updated in DB', chat_id)
        logger.debug('id %s is user %s', chat_id, serialize_user(user))
        return user

    def update_user_from_tg_user_obj(self, tg_user):
        chat_id = tg_user.id
        user_data = {
            'username': tg_user.username,
            'first_name': tg_user.first_name,
            'last_name': tg_user.last_name,
            'is_bot': tg_user.is_bot,
            'language_code': tg_user.language_code,
        }
        user = self.update_user(chat_id, **user_data)
        return user

    def get_user(self, chat_id):
        return self._user_st.find_one({'chat_id': chat_id})

    def get_users(self):
        return self._user_st.find()



import logging

import settings


def get_logger():
    format_ = '%(asctime)s: %(message)s'
    logging.basicConfig(format='%(asctime)s: %(message)s')

    logger = logging.getLogger(__name__)
    logger.setLevel(settings.log_level)

    file_handler = logging.FileHandler(settings.log_path)
    file_handler.setFormatter(logging.Formatter(format_))
    logger.addHandler(file_handler)

    return logger



#!/usr/bin/env python3
from setuptools import setup


with open('README.md') as f:
    long_description = f.read()


setup(
    name='meetg',
    version='0.2.1',
    packages=['meetg'],
    scripts=['bin/meetg-admin'],
    description='Framework for creating Telegram bots',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/meequz/meetg',
    author='Mikhail Varantsou',
    license='LGPL-3.0',
    author_email='meequz@gmail.com',
    install_requires=['python-telegram-bot', 'pymongo'],
    keywords='telegram bot framework',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    python_requires='>=3.5',
)



import time

import telegram
from telegram.ext import Updater

import settings
from meetg.utils import serialize_user
from meetg.loging import get_logger


logger = get_logger()


class BaseBot:
    """Common Telegram bot logic"""

    def __init__(self, db, mock=False):
        self._mock = mock
        if not mock:
            self._updater = Updater(settings.tg_api_token, use_context=True)
            self._tgbot = self._updater.bot
            self._tgbot_username = self._updater.bot.get_me().username
        self._db = db
        self._init_handlers()

    def _init_handlers(self):
        self._handlers = self.get_handlers()
        if not self._mock:
            for handler in self._handlers:
                self._updater.dispatcher.add_handler(handler)

    def get_handlers(self):
        logger.warning('No handlers found')
        return ()

    def run(self):
        self._updater.start_polling()
        logger.info('%s started', self._tgbot_username)
        self._updater.idle()

    def extract(self, update_obj):
        """Extract commonly used info from update_obj,
        save users in db if they're new, log new message
        """
        chat_id = update_obj.message.chat.id
        msg_id = update_obj.message.message_id
        user = self._db.get_user(chat_id)
        text = update_obj.message.text

        contact = update_obj.message.contact
        location = update_obj.message.location
        text_for_log = repr(text or '')
        tg_user_obj = update_obj.message.from_user

        if user:
            user = self._db.update_user_from_tg_user_obj(tg_user_obj)
        else:
            user = self._db.create_user_from_tg_user_obj(tg_user_obj)

        if contact:
            logger.info('Received contact from %s', chat_id)
            logger.debug(
                'id %s is user %s, contact is %s', chat_id, serialize_user(user),
                contact.phone_number,
            )
        elif location:
            logger.info('Received location from %s', chat_id)
            logger.debug(
                'id %s is user %s, location is %s', chat_id, serialize_user(user),
                location,
            )
        else:
            logger.info('Received message with len %s from %s', len(text or ''), chat_id)
            logger.debug(
                'id %s is user %s, message is: %s', chat_id, serialize_user(user),
                text_for_log[:119],
            )
        return chat_id, msg_id, user, text

    def _log_api_call(self, method_name, kwargs):
        chat_id = kwargs.get('chat_id')
        message_id = kwargs.get('message_id')
        text = repr(kwargs.get('text', ''))

        if method_name == 'send_message':
            logger.info('Sending message with len %s to %s', len(text), chat_id)
            logger.debug('Message: %s', text[:119])
        elif method_name == 'edit_message_text':
            logger.info('Editing message %s in chat %s', message_id, chat_id)
        elif method_name == 'delete_message':
            logger.info('Deleting message %s in chat %s', message_id, chat_id)
        else:
            raise NotImplementedError

    def call_bot_api(self, method_name: str, **kwargs):
        """
        Here is retry logic and logic for dealing with network and load issues
        """
        to_attempt = 5
        success = False
        self._log_api_call(method_name, kwargs)
        method = getattr(self._tgbot, method_name)
        chat_id = kwargs.pop('chat_id', None)

        while to_attempt > 0:
            try:
                resp = method(chat_id=chat_id, **kwargs)
                success = True
                to_attempt = 0
            except telegram.error.NetworkError as exc:
                if 'are exactly the same as' in exc.message:
                    logger.error('Network error: "%s". It\'s ok, nothing to do here', exc.message)
                    success = True
                    to_attempt = 0
                elif "Can't parse entities" in exc.message:
                    logger.error('Network error: "%s". Retrying is pointless', exc.message)
                    to_attempt = 0
                else:
                    logger.error(
                        'Network error: "%s". Waiting 2 seconds then retry', exc.message,
                    )
                    to_attempt -= 1
                    time.sleep(2)
                resp = exc.message
            except telegram.error.TimedOut as exc:
                logger.error('Timed Out. Retrying')
                resp = exc.message
                to_attempt -= 1
            except telegram.error.RetryAfter as exc:
                logger.error(
                    'Telegram says to wait and retry after %s seconds. Doing', exc.retry_after,
                )
                resp = exc.message
                to_attempt -= 2
                time.sleep(exc.retry_after + 1)
            except telegram.error.ChatMigrated as exc:
                logger.error('ChatMigrated error: "%s". Retrying with new chat id', exc)
                resp = exc.message
                chat_id = exc.new_chat_id
                to_attempt -= 1
            except (telegram.error.Unauthorized, telegram.error.BadRequest) as exc:
                logger.error('Error: "%s". Retrying', exc)
                resp = exc.message
                to_attempt -= 2
        logger.debug('Success' if success else 'Fail')
        return success, resp

    def broadcast(self, chat_ids, body, html=False):
        for chat_id in chat_ids:
            self.send_msg(chat_id, body, html=html)
        logger.info('Broadcasted: %s', body[:79])

    def send_msg(self, chat_id, body, msg_id=None, markup=None, html=None, preview=False):
        parse_mode = telegram.ParseMode.HTML if html else None
        success, resp = self.call_bot_api(
            'send_message',
            chat_id=chat_id, text=body, reply_to_message_id=msg_id, reply_markup=markup,
            parse_mode=parse_mode, disable_web_page_preview=not preview,
        )
        return success, resp

    def edit_msg_text(self, chat_id, body, msg_id, preview=False):
        success, resp = self.call_bot_api(
            'edit_message_text',
            text=body, chat_id=chat_id, message_id=msg_id, disable_web_page_preview=not preview,
        )
        return success, resp

    def delete_msg(self, chat_id, msg_id):
        success, resp = self.call_bot_api(
            'delete_message',
            chat_id=chat_id, message_id=msg_id,
        )
        return success, resp
